/**
 * @module @pinia/nuxt
 */
import {
  defineNuxtModule,
  addPlugin,
  isNuxt2,
  addAutoImport,
  createResolver,
} from '@nuxt/kit'

export interface ModuleOptions {
  /**
   * Pinia disables Vuex by default, set this option to `false` to avoid it and
   * use Pinia alongside Vuex (Nuxt 2 only)
   *
   * @default `true`
   */
  disableVuex?: boolean

  /**
   * Array of auto imports to be added to the nuxt.config.js file.
   *
   * @example
   * ```js
   * autoImports: [
   *  // automatically import `defineStore`
   *  'defineStore',
   *  // automatically import `defineStore` as `definePiniaStore`
   *  ['defineStore', 'definePiniaStore',
   * ]
   * ```
   *
   */
  autoImports?: Array<string | [string, string]>
}

export default defineNuxtModule<ModuleOptions>({
  meta: {
    name: 'pinia',
    configKey: 'pinia',
    compatibility: {
      nuxt: '^2.0.0 || ^3.0.0',
      bridge: true,
    },
  },
  defaults: {
    disableVuex: true,
    autoImports: [],
  },
  setup(options, nuxt) {
    const resolver = createResolver(import.meta.url)

    // Disable default Vuex store (Nuxt v2.10+ only)
    if (nuxt.options.features && options.disableVuex && isNuxt2()) {
      nuxt.options.features.store = false
    }

    // Transpile runtime
    nuxt.options.build.transpile.push(resolver.resolve('./runtime'))

    // Make sure we use the mjs build for pinia
    nuxt.options.alias.pinia = 'pinia/dist/pinia.mjs'

    // Add runtime plugin
    if (isNuxt2()) {
      addPlugin(resolver.resolve('./runtime/plugin.vue2'))
    } else {
      addPlugin(resolver.resolve('./runtime/plugin.vue3'))
    }

    // Add auto imports
    const composables = resolver.resolve('./runtime/composables')
    addAutoImport([
      { from: composables, name: 'usePinia' },
      ...options.autoImports!.map((imports) =>
        typeof imports === 'string'
          ? { from: composables, name: imports }
          : { from: composables, name: imports[0], as: imports[1] }
      ),
    ])
  },
})
